%bcond_with bootstrap

%global nodejs_major 18
%global nodejs_minor 14
%global nodejs_patch 2
%global nodejs_release 1
%global nodejs_abi %{nodejs_major}.%{nodejs_minor}
# nodejs_soversion - from NODE_MODULE_VERSION in src/node_version.h
%global nodejs_soversion 108
%global nodejs_version %{nodejs_major}.%{nodejs_minor}.%{nodejs_patch}
%global nodejs_datadir %{_datarootdir}/nodejs

# v8 - from deps/v8/include/v8-version.h
%global v8_major 10
%global v8_minor 2
%global v8_build 154
%global v8_patch 26
%global v8_version %{v8_major}.%{v8_minor}.%{v8_build}.%{v8_patch}
%global v8_release %{nodejs_major}.%{nodejs_minor}.%{nodejs_patch}.%{nodejs_release}

# npm - from deps/npm/package.json
%global npm_version 9.5.0
%global npm_release %{nodejs_major}.%{nodejs_minor}.%{nodejs_patch}.%{nodejs_release}

%bcond_without npm

# ICU - from tools/icu/current_ver.dep
%global icu_major 72
%global icu_minor 1
%global icu_version %{icu_major}.%{icu_minor}
%global icudatadir %{nodejs_datadir}/icudata
%{!?little_endian: %global little_endian %(%{python3} -c "import sys;print (0 if sys.byteorder=='big' else 1)")}

%global openssl30_minimum 3.0.2
%global openssl_fips_configure --openssl-is-fips
%global ssl_configure --shared-openssl %{openssl_fips_configure}

# zlib - from deps/zlib/zlib.h
%global zlib_version 1.2.13
# c-ares - from deps/cares/include/ares_version.h
%global c_ares_version 1.18.1
# llhttp - from deps/llhttp/include/llhttp.h
%global llhttp_version 6.0.10
# libuv - from deps/uv/include/uv/version.h
%global libuv_version 1.44.2
# nghttp2 - from deps/nghttp2/lib/includes/nghttp2/nghttp2ver.h
%global nghttp2_version 1.51.0
# punycode - from lib/punycode.js
%global punycode_version 2.1.0
# uvwasi - from deps/uvwasi/include/uvwasi.h
%global uvwasi_version 0.0.14
# histogram_c - assumed from timestamps::
%global histogram_version 0.9.7

%if %{with bootstrap}
%bcond_without bundled
%else
%bcond_with bundled
%endif

%bcond_without bundled_zlib

%define _lto_cflags %{nil}
%global __python %{python3}



Summary: JavaScript runtime
Name: nodejs
Version: %{nodejs_version}
Release: %{nodejs_release}%{?dist}
License: MIT and ASL 2.0 and ISC and BSD
Group: Development/Languages
URL: http://nodejs.org/
Source0: node-v%{nodejs_version}-stripped.tar.gz
Source1: npmrc
Source2: btest402.js
Source3: https://github.com/unicode-org/icu/releases/download/release-%{icu_major}-%{icu_minor}/icu4c-%{icu_major}_%{icu_minor}-data-bin-b.zip
Source4: https://github.com/unicode-org/icu/releases/download/release-%{icu_major}-%{icu_minor}/icu4c-%{icu_major}_%{icu_minor}-data-bin-l.zip
Patch3000: 0001-Disable-running-gyp-on-shared-deps.patch

BuildRequires: make, gcc >= 8.3.0, gcc-c++ >= 8.3.0, jq, chrpath, ninja-build
BuildRequires: python%{python3_pkgversion}-devel, python%{python3_pkgversion}-setuptools, python%{python3_pkgversion}-jinja2, python-unversioned-command
BuildRequires: libatomic, systemtap-sdt-devel, unzip, openssl >= %{openssl30_minimum}, openssl-devel >= %{openssl30_minimum}

%if %{with bundled_zlib}
Provides: bundled(zlib) = %{zlib_version}
%else
BuildRequires: zlib-devel
%endif
BuildRequires: brotli-devel

%if %{with bundled}
Provides:      bundled(libuv) = %{libuv_version}
%else
BuildRequires: libuv-devel >= %{libuv_version}
Requires:      libuv >= %{libuv_version}
%endif

%if %{with npm}
BuildRequires: nodejs-packaging
%endif

Requires: ca-certificates
Requires: nodejs-libs = %{version}-%{release}

Provides: bundled(nghttp2) = %{nghttp2_version}
Provides: bundled(llhttp) = %{llhttp_version}
Provides: nodejs(abi) = %{nodejs_abi}
Provides: nodejs(abi%{nodejs_major}) = %{nodejs_abi}
Provides: nodejs(engine) = %{nodejs_version}
Provides: nodejs-punycode = %{punycode_version}
Provides: npm(punycode) = %{punycode_version}
Provides: bundled(c-ares) = %{c_ares_version}
Provides: bundled(v8) = %{v8_version}
Provides: bundled(icu) = %{icu_version}
Provides: bundled(uvwasi) = %{uvwasi_version}
Provides: bundled(histogram) = %{histogram_version}

Recommends: npm >= %{npm_version}-%{npm_release}%{?dist}
Recommends: nodejs-full-i18n = %{version}-%{release}

%description
Node.js is a platform built on Chrome's JavaScript runtime
for easily building fast, scalable network applications.
Node.js uses an event-driven, non-blocking I/O model that
makes it lightweight and efficient, perfect for data-intensive
real-time applications that run across distributed devices.

%package devel
Summary: JavaScript runtime - development headers
Group: Development/Languages
Requires: %{name} = %{nodejs_version}-%{nodejs_release}%{?dist}
Requires: %{name}-libs = %{nodejs_version}-%{nodejs_release}%{?dist}
Requires: openssl-devel
%if !%{with bundled_zlib}
Requires: zlib-devel
%endif
Requires: brotli-devel
%if %{with npm}
Requires: nodejs-packaging
%endif

%if %{without bundled}
Requires: libuv-devel
%endif

%description devel
Development headers for the Node.js JavaScript runtime.


%package libs
Summary: Node.js and v8 libraries

Provides: libv8.so.%{v8_major}()(64bit) = %{v8_version}
Provides: libv8_libbase.so.%{v8_major}()(64bit) = %{v8_version}
Provides: libv8_libplatform.so.%{v8_major}()(64bit) = %{v8_version}
Provides: v8 = %{v8_version}-%{nodejs_release}%{?dist}
Provides: v8 = %{v8_version}-%{nodejs_release}%{?dist}

%description libs
Libraries to support Node.js and provide stable v8 interfaces.

%package full-i18n
Summary: Non-English locale data for Node.js
Requires: %{name} = %{nodejs_version}-%{nodejs_release}%{?dist}

%description full-i18n
Optional data files to provide full-icu support for Node.js. Remove this
package to save space if non-English locales are not needed.

%package -n v8-devel
Summary: v8 - development headers
Version: %{v8_version}
Release: %{v8_release}%{?dist}
Requires: %{name}-devel = %{nodejs_version}-%{nodejs_release}%{?dist}
Requires: %{name}-libs = %{nodejs_version}-%{nodejs_release}%{?dist}

%description -n v8-devel
Development headers for the v8 runtime.

%if %{with npm}
%package -n npm
Summary: Node.js Package Manager
Version: %{npm_version}
Release: %{npm_release}%{?dist}
Requires: nodejs = %{nodejs_version}-%{nodejs_release}%{?dist}
Recommends: nodejs-docs = %{nodejs_version}-%{nodejs_release}%{?dist}
Provides: npm(npm) = %{npm_version}

%description -n npm
npm is a package manager for node.js. You can use it to install and publish
your node programs. It manages dependencies and does other cool stuff.
%endif

%package docs
Summary: Node.js API documentation
Group: Documentation
BuildArch: noarch

%description docs
The API documentation for the Node.js JavaScript runtime.



%prep
%autosetup -p1 -n node-v%{nodejs_version}

%if !%{with bundled_zlib}
rm -rf deps/zlib
%endif

rm -rf deps/brotli
rm -rf deps/v8/third_party/jinja2
rm -rf tools/inspector_protocol/jinja2

pfiles=( $(grep -rl python) )
%py3_shebang_fix ${pfiles[@]}



%build
%global optflags %(echo %{optflags} | sed 's/-g /-g1 /')

export CC='%{__cc}'
export CXX='%{__cxx}'
export NODE_GYP_FORCE_PYTHON=%{python3}

extra_cflags=(
    -D_LARGEFILE_SOURCE
    -D_FILE_OFFSET_BITS=64
    -DZLIB_CONST
    -fno-delete-null-pointer-checks
    -O3
)
export CFLAGS="%{optflags} ${extra_cflags[*]}" CXXFLAGS="%{optflags} ${extra_cflags[*]}"
export LDFLAGS="%{build_ldflags}"

%{python3} configure.py \
           --ninja \
           --enable-lto \
           --prefix=%{_prefix} \
           --shared \
           --libdir=%{_lib} \
           %{ssl_configure} \
           %{!?with_bundled_zlib:--shared-zlib} \
           --shared-brotli \
           %{!?with_bundled:--shared-libuv} \
           %{?with_bundled:--without-dtrace}%{!?with_bundled:--with-dtrace} \
           --with-intl=small-icu \
           --with-icu-default-data-dir=%{icudatadir} \
           --without-corepack \
           %{!?with_npm:--without-npm} \
           --openssl-use-def-ca-store

%ninja_build -C out/Release



%install
rm -rf %{buildroot}

mv out/Release/lib/libnode.so.%{nodejs_soversion} out/Release/
./tools/install.py install %{buildroot} %{_prefix}

chmod 0755 %{buildroot}/%{_bindir}/node
chrpath --delete %{buildroot}%{_bindir}/node

ln -s libnode.so.%{nodejs_soversion} %{buildroot}%{_libdir}/libnode.so

for header in %{buildroot}%{_includedir}/node/libplatform %{buildroot}%{_includedir}/node/v8*.h; do
    header=$(basename ${header})
    ln -s ./node/${header} %{buildroot}%{_includedir}/${header}
done
ln -s ./node/cppgc %{buildroot}%{_includedir}/cppgc
for soname in libv8 libv8_libbase libv8_libplatform; do
    ln -s libnode.so.%{nodejs_soversion} %{buildroot}%{_libdir}/${soname}.so
    ln -s libnode.so.%{nodejs_soversion} %{buildroot}%{_libdir}/${soname}.so.%{v8_major}
done

mkdir -p %{buildroot}%{_prefix}/lib/node_modules

mkdir -p %{buildroot}%{_pkgdocdir}/html
cp -pr doc/* %{buildroot}%{_pkgdocdir}/html
rm -f %{buildroot}%{_pkgdocdir}/html/nodejs.1

mkdir -p %{buildroot}%{_datadir}/node
cp -p common.gypi %{buildroot}%{_datadir}/node

mv %{buildroot}%{_includedir}/node/config.gypi \
   %{buildroot}%{_includedir}/node/config-%{_arch}.gypi

mv %{buildroot}/%{_datadir}/doc/node/gdbinit %{buildroot}/%{_pkgdocdir}/gdbinit

%if %{with npm}
mkdir -p %{buildroot}%{_mandir} \
         %{buildroot}%{_pkgdocdir}/npm

cp -pr deps/npm/man/* %{buildroot}%{_mandir}/
rm -rf %{buildroot}%{_prefix}/lib/node_modules/npm/man
ln -srf %{buildroot}%{_mandir} %{buildroot}%{_prefix}/lib/node_modules/npm/man

cp -pr deps/npm/docs %{buildroot}%{_pkgdocdir}/npm/
rm -rf %{buildroot}%{_prefix}/lib/node_modules/npm/docs

ln -srf %{buildroot}%{_pkgdocdir}/npm %{buildroot}%{_prefix}/lib/node_modules/npm/docs
%endif

rm -f %{buildroot}/%{_defaultdocdir}/node/lldb_commands.py \
      %{buildroot}/%{_defaultdocdir}/node/lldbinit

%if %{with npm}
find %{buildroot}%{_prefix}/lib/node_modules/npm \
    -not -path "%{buildroot}%{_prefix}/lib/node_modules/npm/bin/*" \
    -executable -type f \
    -exec chmod -x {} \;

chmod 0755 %{buildroot}%{_prefix}/lib/node_modules/npm/node_modules/@npmcli/run-script/lib/node-gyp-bin/node-gyp
chmod 0755 %{buildroot}%{_prefix}/lib/node_modules/npm/node_modules/node-gyp/bin/node-gyp.js

mkdir -p %{buildroot}%{_sysconfdir}
cp %{SOURCE1} %{buildroot}%{_sysconfdir}/npmrc

mkdir -p %{buildroot}%{_prefix}/etc
ln -rsf %{buildroot}%{_sysconfdir}/npmrc %{buildroot}%{_prefix}/etc/npmrc
%endif

mkdir -p %{buildroot}%{icudatadir}
%if 0%{?little_endian}
unzip -d %{buildroot}%{icudatadir} %{SOURCE4} icudt%{icu_major}l.dat
%else
unzip -d %{buildroot}%{icudatadir} %{SOURCE3} icudt%{icu_major}b.dat
%endif



%check
LD_LIBRARY_PATH=%{buildroot}%{_libdir} %{buildroot}/%{_bindir}/node -e "require('assert').equal(process.versions.node, '%{nodejs_version}')"
LD_LIBRARY_PATH=%{buildroot}%{_libdir} %{buildroot}/%{_bindir}/node -e "require('assert').equal(process.versions.v8.replace(/-node\.\d+$/, ''), '%{v8_version}')"
LD_LIBRARY_PATH=%{buildroot}%{_libdir} %{buildroot}/%{_bindir}/node -e "require('assert').equal(process.versions.ares.replace(/-DEV$/, ''), '%{c_ares_version}')"

LD_LIBRARY_PATH=%{buildroot}%{_libdir} %{buildroot}/%{_bindir}/node -e "require(\"assert\").equal(require(\"punycode\").version, '%{punycode_version}')"

%if %{with npm}
LD_LIBRARY_PATH=%{buildroot}%{_libdir} %{buildroot}%{_bindir}/node %{buildroot}%{_bindir}/npm version --json |jq -e '.npm == "%{npm_version}"'
%endif

NODE_PATH=%{buildroot}%{_prefix}/lib/node_modules:%{buildroot}%{_prefix}/lib/node_modules/npm/node_modules LD_LIBRARY_PATH=%{buildroot}%{_libdir} %{buildroot}/%{_bindir}/node --icu-data-dir=%{buildroot}%{icudatadir} %{SOURCE2}



%files
%doc AUTHORS CHANGELOG.md onboarding.md GOVERNANCE.md README.md
%doc %{_mandir}/man1/node.1*
%{_bindir}/node

%if %{without bundled}
%dir %{_usr}/lib/dtrace
%{_usr}/lib/dtrace/node.d

%endif
%dir %{_prefix}/lib/node_modules
%dir %{_datadir}/node
%dir %{_datadir}/systemtap
%dir %{_datadir}/systemtap/tapset
%{_datadir}/systemtap/tapset/node.stp

%files devel
%{_includedir}/node
%{_libdir}/libnode.so
%{_datadir}/node/common.gypi
%{_pkgdocdir}/gdbinit

%files full-i18n
%dir %{icudatadir}
%{icudatadir}/icudt%{icu_major}*.dat

%files libs
%license LICENSE
%{_libdir}/libnode.so.%{nodejs_soversion}
%{_libdir}/libv8.so.%{v8_major}
%{_libdir}/libv8_libbase.so.%{v8_major}
%{_libdir}/libv8_libplatform.so.%{v8_major}
%dir %{nodejs_datadir}/

%files -n v8-devel
%{_includedir}/libplatform
%{_includedir}/v8*.h
%{_includedir}/cppgc
%{_libdir}/libv8.so
%{_libdir}/libv8_libbase.so
%{_libdir}/libv8_libplatform.so

%if %{with npm}
%files -n npm
%{_bindir}/npm
%{_bindir}/npx
%{_prefix}/lib/node_modules/npm
%config(noreplace) %{_sysconfdir}/npmrc
%{_prefix}/etc/npmrc
%ghost %{_sysconfdir}/npmignore
%doc %{_mandir}/man*/
%exclude %doc %{_mandir}/man1/node.1*
%endif

%files docs
%doc doc
%dir %{_pkgdocdir}
%{_pkgdocdir}/html

%if %{with npm}
%{_pkgdocdir}/npm/docs
%endif



%changelog
* Fri Jul 21 2023 Miaojun Dong <zoedong@tencent.com> - 18.14.2-1
- Bump version to 18.14.2

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 18.12.1-6
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 18.12.1-5
- Rebuilt for OpenCloudOS Stream 23

* Thu Feb 09 2023 Miaojun Dong <zoedong@tencent.com> - 18.12.1-4
- add dist to release number

* Mon Jan 9 2023 Zhao Zhen <jeremiazhao@tencent.com> 18.12.1-3
- enabled npm

* Mon Jan 9 2023 Zhao Zhen <jeremiazhao@tencent.com> 18.12.1-2
- modified macro arrange

* Fri Dec 30 2022 Zhao Zhen <jeremiazhao@tencent.com> 18.12.1-1
- initial
